# vim fileencoding=utf-8
# vi:ts=4:et
# $Id: matrixball.py,v 1.2 2004/12/04 05:57:52 etale Exp $
# =====================================================

"""
Matrix Ball Construction
"""
__all__ = [
    'MatrixBall',
    'Numbering',
    ]

from cStringIO import StringIO

# young
from young import Young, Word
from rsk import RSK
import util

class MatrixBall(object):
    def __init__(self, matrix):
        self.matrix = matrix

        self.P = Young()
        self.Q = Young()

        self.setup()

    def setup(self):
        """
        construct a pair of tableaux from a given matrix.
        """
        constructed = Numbering(self.matrix)

        buff_P = [constructed.constructP()]
        buff_Q = [constructed.constructQ()]

        while constructed.has_same_label():

            constructed = Numbering(constructed.get_derived_matrix())

            buff_P.append(constructed.constructP())
            buff_Q.append(constructed.constructQ())

        self.P = Young(buff_P)
        self.Q = Young(buff_Q)

    def __repr__(self):

        buff = []
        buff.append("# tableau P")
        buff.append("%s"%self.P)
        buff.append("# tableau Q")
        buff.append("%s"%self.Q)

        return "\n".join(buff)

    __str__ = __repr__

    def get_A_0(self):
        """
        A_0 is a matrix obtained from A by subtracting 1 from corresponding entry a(x, y) of A.
        """
        # Fulton P. 44

        x, y = self.get_last_position()

        derived_matrix = self.matrix[:]
        derived_matrix[x-1][y-1] -= 1

        return derived_matrix

    def get_last_position(self):
        """Return the position (x, y) from matrix A the position such that
        the x-th row of A is the lowest row that is nonzero and the y-th
        entry of this row is the right-most entry of this row that is
        nonzero.
        """
        # Fulton P. 44

        for i, row in enumerate(reversed(self.matrix)):
            if row.count(0) != len(row):
                # If all entries in a given row is not zero,,,
                for j , entry in enumerate(reversed(row)):
                    # We need the right-most entry, so reversed is necessary.
                    if entry != 0:
                        return (len(self.matrix) - i, len(row) - j)
        raise IndexError, "failed to find the last position from matrix : %s"%self.matrix

    def get_tableau_pair(self):
        """
        Return the tableau pair corresponding to a matrix.
        """
        # Fulton P. 46

        mb = MatrixBall(self.matrix[:])
        xs = []
        ys = []

        while True:
            try:
                x, y = mb.get_last_position()
                xs.append(y)
                ys.append(x)

                mb = MatrixBall(mb.get_A_0())
            except IndexError, e:
                break

        rsk = RSK()
        rsk.set_array(xs, ys)
        rsk.setup()
        return rsk


class Numbering(object):
    """this class stores numbered balls.
    """
    # Based on a given matrix, this class creates a matrix with numbered
    # balls and a piar (P, Q) = (P(A), Q(A)) of tableaux.
    #
    # The algorithm is described in Fulton P.43-.

    # Fulton P.43

    def __init__(self, matrix):
        self.matrix = matrix

        # box is a list of list that holds numbered balls.
        self.box = [[None] * len(self.matrix[i]) for i in range(len(self.matrix))]

        # If self.upper_limit is n, balls are numbered
        # from 1 to (n-1).
        self.upper_limit = 1

        self.P = ()
        self.Q = ()

        self.numbering()
        self.set_duplicate()

    def __repr__(self):
        # If a given box is [[[1], [2, 3]], [[2], [4]], [[3, 4, 5], [6]]] ,
        # the box is represented as
        #
        #      1      |    2 3
        # -----------------------
        #      2      |     4
        # -----------------------
        #    3 4 5    |     6

        WIDTH = 12

        buff = StringIO()
        for row in self.box:
            inner_buff = []
            for col in row:

                if col is None:
                    inner_buff.append(' '*WIDTH)
                else:
                    inner_buff.append(' '.join(map(str, col)).center(WIDTH))
            else:
                buff.write('|'.join(inner_buff))
                buff.write('\n')
            buff.write("-" * (WIDTH * len(row) + len(row) -1) + "\n")

        return buff.getvalue()

    __str__ = __repr__

    def set_duplicate(self):
        """
        If there are any duplicated balls, mark them as `duplicated'.
        """
        # cache stores information about where numbered balls are located.
        cache      = {}

        # duplicated stores information about numbered balls which are
        # located more than one place.
        duplicated = {} 

        # Example:
        # If self.box == [[[1], [2, 3]], [[2], [4]], [[3, 4, 5], [6]]],
        # cache  ==
        # {1: [(0, 0)], 2: [(0, 1), (1, 0)], 3: [(0, 1), (2, 0)], 4: [(1, 1), (2, 0)], 5: [(2, 0)], 6: [(2, 1)]}
        # duplicated ==
        # {2: [(0, 1), (1, 0)], 3: [(0, 1), (2, 0)], 4: [(1, 1), (2, 0)]}

        for i in range(len(self.box)):
            for j in range(len(self.box[i])):
                # self.box[i] == [[1], [2, 3]]
                if self.box[i][j] is None:
                    continue

                for num in self.box[i][j]:
                    # self.box[i][j] == [2, 3]
                    cache.setdefault(num, []).append((i, j))

        for k, v in cache.iteritems():
            if len(v) > 1:
                # found duplicated balls
                duplicated[k] = v

        self.duplicated = duplicated

    def has_same_label(self):
        """Test if there are any balls in a box with the same label.
        """
        # If this condition is evaluated as True, you can't create a derived matrix.(P. 44)

        if self.duplicated:
            return True
        else:
            return False

    def get_derived_matrix(self):
        """
        Create a derived matrix from an original one.
        This matrix can be used to create a new instance of Numbering class.

        """
        # ``derived'' matrix is defined in Fulton P. 44.


        derived_matrix = [[0] * len(self.matrix[i]) for i in range(len(self.matrix))]

        for key, val in self.duplicated.iteritems():
            for P, Q in util.pairwise(val):

                if P[1] > Q[1]:
                    P, Q = Q, P

                derived_matrix[P[0]][Q[1]] += 1

        return derived_matrix

    def contain(self, num, matrix):
        """
        Test if a given matrix contains a given number.
        """
        # Examples :
        # contain(2, [[1], [2, 3]]2) => True
        # contain(3, [None, [2, 3]]) => True
        # contain(4, [None, [2, 3]]) => False

        for row in matrix:
            if row is None:
                continue
            elif num in row:
                return True
        else:
            return False

    def constructP(self):
        """
        P is a sequence which lists the left-most columns where each number
        occurs in a matrix with numbered balls.
        """

        def _constructP():
            for num in range(self.upper_limit):
                for i, line in enumerate(util.transpose_generator(self.box)):
                    result = self.contain(num+1, line)
                    if result:
                        yield i+1
                        break

        self.P = list(_constructP())
        return self.P

    def constructQ(self):
        """
        Q is a sequence which lists the top-most rows where each number
        occurs in a matrix with numbered balls.
        """
        def _constructQ():
            for num in range(self.upper_limit):
                for i, line in enumerate(self.box):
                    result = self.contain(num+1, line)
                    if result:
                        yield i+1
                        break

        self.Q = list(_constructQ())

        return self.Q

    def numbering(self):
        """Put numbered balls into the matrix from NorthWest to SouthEast.
        """
        # Carry out the numbering

        # the algorithm of this method described in Fulton P. 43-.

        # self.matrix is something like:
        # [[1, 2], [1, 1], [3, 1]]

        # base_number is the smallest number to be pushed into the matrix.
        base_number = 0

        for i, row in enumerate(self.matrix):
            for j, col in enumerate(row):

                # XXX
                # What happens if all elements in row is 0. like,
                # row = [0, 0]
                if self.matrix[i][j] == 0:
                    # row = [0, 1]
                    continue

                candidates = [0]
                for row in self.box[:i+1]:
                    for entry in reversed(row[:j+1]):
                        if entry is not None:
                            candidates.append(max(entry))
                            break

                base_number = max(candidates)+1

                # col is the number of balls to be pushed into the box.
                # For example, if base_number is 5 and col is 3,
                # self.box[i][j] has numbered balls {5,6,7}

                self.box[i][j] = range(base_number, base_number + col)
        else:
            self.upper_limit = base_number + col

