# vim fileencoding=utf-8
# vi:ts=4:et
# $Id: test_matrixball.py,v 1.2 2004/12/04 05:57:52 etale Exp $
# =====================================================

"""
unittest for matrixball.py
"""
import unittest
from test import test_support

from young import *

class TestMBBasicOperation(unittest.TestCase):
    def test_construct_new_matrix(self):
        # This example is taken from Fulton P. 43
        # Create A^{flat} from a matrix A

        # test of only Numbering

        A = [[1,2], [1,1], [3,1]]
        b = Numbering(A)

        # b.box = A^{(1)}
        self.assertEqual(b.box,
                [[[1], [2, 3]], [[2], [4]], [[3, 4, 5], [6]]])

        self.assertEqual(b.has_same_label(), True)

        # construct first row of P
        self.assertEqual(b.constructP(), [1, 1, 1, 1, 1, 2])
        # construct first row of Q
        self.assertEqual(b.constructQ(), [1, 1, 1, 2, 3, 3])
    
        # b.get_derived_matrix() = A^{flat}
        self.assertEqual(b.get_derived_matrix(),
                [[0, 0], [0, 1], [0, 2]])


    def test_construct_new_matrix2(self):
        # This example is taken from Fulton P. 43

        # test of both Numbering and MatrixBall

        A  = [[1,2], [1,1], [3,1]]
        mb = MatrixBall(A)

        self.assertEqual(mb.P,
                [[1, 1, 1, 1, 1, 2], [2, 2, 2]])
        self.assertEqual(mb.Q,
                [[1, 1, 1, 2, 3, 3], [2, 3, 3]])

        constructed = Numbering(mb.matrix)

        # A^{(1)}
        self.assertEqual(constructed.box,
                [[[1], [2, 3]], [[2], [4]], [[3, 4, 5], [6]]])

        self.assertEqual(constructed.has_same_label(), True)

        if constructed.has_same_label():
            self.assertEqual(constructed.get_derived_matrix(),
                    [[0, 0], [0, 1], [0, 2]])

    def test_construct_new_matrix3(self):
        # P. 44

        # Create A^{(2)} from a matrix A^{flat}

        A = [[0, 0], [0, 1], [0, 2]]    # A^{flat}
        b = Numbering(A)

        # Test if A^{(2)} is created correctly
        self.assertEqual(b.box, [[None, None], [None, [1]], [None, [2,3]]])

        self.assertEqual(b.has_same_label(), False)

        self.assert_(b.constructP() == [2,2,2] == b.P)
        self.assert_(b.constructQ() == [2,3,3] == b.Q)

class TestMB(unittest.TestCase):

    def test_ex3(self):
        # Exercise 3(P. 46)

        A  = [[1, 2, 1],[2, 2, 0],[1, 1, 1]]
        mb = MatrixBall(A)

        xs = []
        ys = []

        while True:
            try:
                x, y = mb.get_last_position()
                xs.append(y)
                ys.append(x)

                new_A = mb.get_A_0()

                mb = MatrixBall(new_A)
            except IndexError, e:
                break

        self.assertEqual(xs, [3, 2, 1, 2, 2, 1, 1, 3, 2, 2, 1])
        self.assertEqual(ys, [3, 3, 3, 2, 2, 2, 2, 1, 1, 1, 1])

        rsk = RSK()
        rsk.set_array(xs, ys)
        rsk.setup()

        self.assertEqual(rsk.P, 
                Young([[1, 1, 1, 1, 2, 3, 3], [2, 2, 2], [3]]))

        self.assertEqual(rsk.Q,
                    Young([[1, 1, 1, 1, 2, 2, 3], [2, 2, 2], [3]]))


    def test_ex3_simple(self):
        # Exercise 3(P. 46)
        # using a get_tableau_pair method directly.

        A  = [[1, 2, 1],[2, 2, 0],[1, 1, 1]]
        mb = MatrixBall(A)
        rsk = mb.get_tableau_pair()

        self.assertEqual(rsk.P, 
                Young([[1, 1, 1, 1, 2, 3, 3], [2, 2, 2], [3]]))

        self.assertEqual(rsk.Q,
                    Young([[1, 1, 1, 1, 2, 2, 3], [2, 2, 2], [3]]))


    def test_ex3_2(self):
        # yet another demonstration of Ex.3(P. 46)
        # using a different matrix.

        A  = [[1, 2], [1, 1], [3, 1]]
        mb = MatrixBall(A)

        xs = []
        ys = []

        while True:
            try:
                x, y = mb.get_last_position()
                xs.append(y)
                ys.append(x)

                new_A = mb.get_A_0()

                mb = MatrixBall(new_A)
            except IndexError, e:
                break

        self.assertEqual(new_A, [[0, 0], [0, 0], [0, 0]])

        self.assertEqual(xs, [2, 1, 1, 1, 2, 1, 2, 2, 1])
        self.assertEqual(ys, [3, 3, 3, 3, 2, 2, 1, 1, 1])

        y = Young()
        for w in reversed(xs):
            y.bump(w)

        self.assertEqual(y, Young([[1, 1, 1, 1, 1, 2], [2, 2, 2]]))

        rsk = RSK()
        rsk.set_array(xs, ys)
        rsk.setup()

        self.assertEqual(rsk.P,
                Young([[1, 1, 1, 2, 3, 3], [2, 3, 3]]))

        self.assertEqual(rsk.Q,
                Young([[1, 1, 1, 1, 1, 2], [2, 2, 2]]))

    def test_ex3_2_simple(self):
        # yet another demonstration of Ex.3(P. 46)
        # using a get_tableau_pair method directly.
        A  = [[1, 2], [1, 1], [3, 1]]
        mb = MatrixBall(A)
        rsk = mb.get_tableau_pair()

        self.assertEqual(rsk.P, 
                Young([[1, 1, 1, 2, 3, 3], [2, 3, 3]]))

        self.assertEqual(rsk.Q,
                Young([[1, 1, 1, 1, 1, 2], [2, 2, 2]]))

    def test_symmetry(self):
        # P. 46
        # The matrix A is symmetric if and only if P = Q

        A    = [[1,2,1], [2,2,0], [1,0,1]]       # symmetric matrix
        mb   = MatrixBall(A)
        self.assertEqual(mb.P, mb.Q)
        self.assert_(mb.P == mb.Q == Young([[1, 1, 1, 1, 2, 3], [2, 2, 2], [3]]))

    def test_ex4(self):
        # P. 47

        A    = [[1,2,1],[2,2,0], [1,0,1]]        # symmetric matrix
        mb   = MatrixBall(A)
        ball = Numbering(A)

        if ball.has_same_label():
            A_flat = ball.get_derived_matrix()
        else:
            # Since We can't create A_flat matrix from A,
            # ignore it.
            return

        # length of first row of P = len(mb.P[0])
        self.assertEqual(len(mb.P[0]), util.trace(A) + util.trace(A_flat))

    def test_prop1(self):
        # P. 44
        # Proposition 1

        # If the matrix A corresponds to a two-rowed array omega, then
        # (P(A), Q(A)) = (P(omega), Q(omega))

        # relation between MatrixBall and RSK.

        A = [[1,2], [1,1], [3,1]]
        mb = MatrixBall(A)

        self.assertEqual(mb.P, [[1, 1, 1, 1, 1, 2], [2, 2, 2]])
        self.assertEqual(mb.Q, [[1, 1, 1, 2, 3, 3], [2, 3, 3]])

        # w1 & w2 is a two-rowed array.
        r = RSK()
        w1 = (1,1,1,2,2,3,3,3,3)
        w2 = (1,2,2,1,2,1,1,1,2)
        r.set_array(w1, w2)
        r.setup()

        self.assertEqual(r.P, mb.P)
        self.assertEqual(r.Q, mb.Q)


def test_main(verbose=None):
    testclasses = (
            TestMBBasicOperation,
            TestMB,
            )

    test_support.run_unittest(*testclasses)

if __name__ == '__main__':
    test_main()

